package jdk.management.resource.internal.inst;

import jdk.internal.instrumentation.InstrumentationMethod;
import jdk.management.resource.ResourceRequest;
import jdk.management.resource.ResourceRequestDeniedException;
import java.io.IOException;
import jdk.management.resource.internal.ApproverGroup;
import jdk.management.resource.internal.ResourceIdImpl;
import java.io.FileDescriptor;
import jdk.internal.instrumentation.TypeMapping;
import jdk.internal.instrumentation.InstrumentationTarget;

@InstrumentationTarget("java.net.SocketInputStream")
@TypeMapping(from = "jdk.management.resource.internal.inst.SocketInputStreamRMHooks$AbstractPlainSocketImpl", to = "java.net.AbstractPlainSocketImpl")
public final class SocketInputStreamRMHooks
{
    private SocketInputStreamRMHooks.AbstractPlainSocketImpl impl;
    
    public SocketInputStreamRMHooks() {
        this.impl = null;
    }
    
    @InstrumentationMethod
    private int socketRead(FileDescriptor fileDescriptor, byte[] array, int n, int min, int n2) throws IOException {
        if (min < 0) {
            return this.socketRead(fileDescriptor, array, n, min, n2);
        }
        ResourceIdImpl of = ResourceIdImpl.of(Integer.valueOf(this.impl.localport));
        ResourceRequest approver = ApproverGroup.SOCKET_READ_GROUP.getApprover(this);
        long max;
        try {
            max = Math.max(approver.request((long)min, of), 0L);
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        min = Math.min(min, (int)max);
        int max2 = 0;
        int socketRead;
        try {
            socketRead = this.socketRead(fileDescriptor, array, n, min, n2);
            max2 = Math.max(socketRead, 0);
        }
        finally {
            approver.request(-(max - (long)max2), of);
        }
        return socketRead;
    }
    
    static class AbstractPlainSocketImpl
    {
        protected int localport;
    }
}
